package com.redxun.knowledge.album.controller;

import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.knowledge.album.entity.dao.AlbumCategory;
import com.redxun.knowledge.album.entity.vo.AlbumCategoryVo;
import com.redxun.knowledge.album.service.AlbumCategoryServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import oracle.jdbc.proxy.annotation.Post;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/album/category")
@Api(tags = "业务--专辑分类表")
@ClassDefine(title = "业务--专辑分类表", alias = "AlbumCategoryController", path = "/api-knowledge/album/category", packages = "album", packageName = "子系统名称")
public class AlbumCategoryController extends BaseController<AlbumCategory> {

    @Autowired
    AlbumCategoryServiceImpl albumCategoryService;


    @Override
    public BaseService getBaseService() {
        return albumCategoryService;
    }

    @Override
    public String getComment() {
        return "业务--专辑分类表";
    }

    @ApiOperation("根据专辑分类ID获取分类详细信息")
    @GetMapping("info")
    public JsonResult info(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumCategoryVo albumCategoryVo = albumCategoryService.info(pkId);
        return jsonResult.setData(albumCategoryVo);
    }

    @ApiOperation("查询对应级别的专辑分类列表")
    @GetMapping("listOflevel")
    public JsonResult listOflevel(@RequestParam("level") Integer level){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<AlbumCategoryVo> albumCategoryVoList = albumCategoryService.listOflevel(level);
        return jsonResult.setData(albumCategoryVoList);
    }

    @ApiOperation("删除专辑分类")
    @Override
    public JsonResult del(String ids) {
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String del = albumCategoryService.del(ids);
        if ("1".equals(del)){
            jsonResult.setMessage("该分类下还有子分类,删除失败");
            return jsonResult;
        }
        else if ("-1".equals(del)){
            return jsonResult.setMessage("删除成功!");
        } else {
            jsonResult.setMessage("您要删除的分类已关联" + del);
            return jsonResult;
        }
    }

    @ApiOperation("编辑专辑分类")
    @PostMapping("edit")
    public JsonResult edit(@RequestBody AlbumCategory albumCategory){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String data = albumCategoryService.edit(albumCategory);
        return jsonResult.setData(data);
    }

    @ApiOperation("上升、下降方式调整专辑分类顺序")
    @PostMapping("adjustDirection")
    public JsonResult adjustDirection(@RequestParam("pkId") String pkId,
                                      @RequestParam("direction") Integer direction){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumCategoryService.adjustDirection(pkId,direction);
        if (direction == 1){
            return jsonResult.setMessage("上升成功");
        } else {
            return jsonResult.setMessage("下降成功");
        }
    }

    @ApiOperation("根据父Id查询子分类集合(创建专辑时PC端使用 懒加载模式)")
    @GetMapping("lazyLoad")
    public JsonResult lazyLoad(@RequestParam(value = "pkId",defaultValue = "0") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<AlbumCategory> albumCategoryList = albumCategoryService.lazyLoad(pkId);
        return jsonResult.setData(albumCategoryList);
    }
}

