package com.redxun.knowledge.album.controller;

import com.google.j2objc.annotations.AutoreleasePool;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.knowledge.album.entity.dao.AlbumApprovalNode;
import com.redxun.knowledge.album.entity.dao.AlbumInfo;
import com.redxun.knowledge.album.entity.dao.AlbumYelpApprovalNode;
import com.redxun.knowledge.album.entity.dto.AlbumApprovalNodeDto;
import com.redxun.knowledge.album.entity.dto.AlbumApprovalRemark;
import com.redxun.knowledge.album.entity.dto.AlbumInfoDto;
import com.redxun.knowledge.album.entity.vo.*;
import com.redxun.knowledge.album.service.AlbumApprovalNodeServiceImpl;
import com.redxun.knowledge.album.service.AlbumApproveServiceImpl;
import com.redxun.knowledge.album.service.AlbumInfoServiceImpl;
import com.redxun.knowledge.album.service.AlbumVersionServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.signature.qual.ClassGetName;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/album")
@Api(tags = "业务--专辑基本信息表")
@ClassDefine(title = "业务--专辑基本信息表", alias = "AlbumInfoController", path = "/api-knowledge/album", packages = "album", packageName = "子系统名称")
public class AlbumInfoController extends BaseController<AlbumInfo> {

    @Autowired
    AlbumInfoServiceImpl albumInfoService;

    @Autowired
    private AlbumApprovalNodeServiceImpl albumApprovalNodeService;

    @Autowired
    private AlbumApproveServiceImpl albumApprovalService;

    @Autowired
    private AlbumVersionServiceImpl albumVersionService;


    @Override
    public BaseService getBaseService() {
        return albumInfoService;
    }

    @Override
    public String getComment() {
        return "业务--专辑基本信息表";
    }

    @ApiOperation("PC端新增知识专辑")
    @PostMapping("create")
    public JsonResult create(@RequestBody AlbumInfoDto albumInfoDto){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        String pkId = albumInfoService.create(albumInfoDto);
        return jsonResult.setData(pkId);
    }

    @ApiOperation("根据知识专辑ID获取知识专辑详情数据")
    @GetMapping("detail")
    public JsonResult detail(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumInfoDetailVo albumInfoDetailVo = albumInfoService.detail(pkId);
        return jsonResult.setData(albumInfoDetailVo);
    }

    @ApiOperation("根据知识专辑版本ID获取知识专辑版本的详情数据")
    @GetMapping("version")
    public JsonResult version(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumVersionDetailVo albumVersionDetailVo = albumInfoService.version(pkId);
        return jsonResult.setData(albumVersionDetailVo);
    }

    @ApiOperation("批量更新审核流程节点数据")
    @PostMapping("batchUpdateApproveFlows")
    public JsonResult batchUpdateApproveFlows(@RequestBody List<AlbumApprovalNodeDto> albumApprovalNodeDtoList){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumInfoService.batchUpdateApproveFlows(albumApprovalNodeDtoList);
        return jsonResult.setData("批量添加成功!");
    }
    
    @ApiOperation("获取当前审核流程列表数据")
    @GetMapping("approveFlows")
    public JsonResult approveFlows(@RequestParam("type") Integer type){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        if (type == 1){
            List<AlbumApproverNodeVo> albumApproverNodeVos = albumInfoService.approveFlowsAlbumApprovalNode();
            return jsonResult.setData(albumApproverNodeVos);
        }
        else if (type == 2){
            List<AlbumApproverNodeVo> albumApproverNodeVos = albumInfoService.approveFlowsAlbumYelpApprovalNode();
            return jsonResult.setData(albumApproverNodeVos);
        } else {
            return jsonResult.setMessage("类型错误");
        }
    }

    @ApiOperation("获取专辑审核流程模板")
    @GetMapping("approvalTemplate")
    public JsonResult approvalTemplate(){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<Map<String,Object>> result = albumApprovalNodeService.approvalTemplate();
        return jsonResult.setData(result);
    }

    @ApiOperation("对知识专辑审核中的版本进行审核")
    @PostMapping("approve")
    public JsonResult approve(@RequestBody AlbumApprovalRemark albumApprovalRemark){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        albumApprovalService.approve(albumApprovalRemark);
        return jsonResult.setMessage("审核成功!");

    }

    @ApiOperation("PC端根据知识专辑ID获取审批流程列表数据")
    @GetMapping("approvals")
    public JsonResult approvals(@RequestParam("albumId") String albumId,
                                @RequestParam(value = "versionId",required = false) String versionId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumApproveFlowVo albumApproveFlowVoList = albumApprovalService.approvals(albumId,versionId);
        return jsonResult.setData(albumApproveFlowVoList);
    }

    @ApiOperation("PC端根据知识专辑ID获取知识专辑详情")
    @GetMapping("info")
    public JsonResult info(@RequestParam("pkId") String pkId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        AlbumInfoDetailPcVo albumInfoDetailPcVo = albumInfoService.info(pkId);
        return jsonResult.setData(albumInfoDetailPcVo);
    }

    @ApiOperation("PC端根据知识专辑ID和版本ID获取知识专辑目录及目录下知识")
    @GetMapping("catalogue")
    public JsonResult catalogue(@RequestParam("albumId") String albumId,@RequestParam(value = "versionId",required = false) String versionId){
        JsonResult jsonResult = JsonResult.getSuccessResult("");
        List<AlbumCataloguePcVo> albumCatalogueVoList = albumVersionService.catalogue(albumId,versionId);
        return jsonResult.setData(albumCatalogueVoList);
    }

}

