package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.StringUtils;
import com.redxun.knowledge.album.entity.dao.*;
import com.redxun.knowledge.album.entity.dto.AlbumApprovalRemark;
import com.redxun.knowledge.album.entity.enums.AlbumVersionApproveEnum;
import com.redxun.knowledge.album.entity.vo.AlbumApproveFlowVo;
import com.redxun.knowledge.album.entity.vo.AlbumApproveListVo;
import com.redxun.knowledge.album.entity.vo.AlbumApproveVo;
import com.redxun.knowledge.album.mapper.*;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.utils.PageListUtils;
import org.apache.catalina.valves.AbstractAccessLogValve;
import org.owasp.esapi.util.CollectionsUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.xml.crypto.dsig.keyinfo.RetrievalMethod;
import java.util.*;
import java.util.stream.Collectors;

/**
 * [业务--专辑审批表]业务服务类
 */
@Service
public class AlbumApproveServiceImpl extends SuperServiceImpl<AlbumApproveMapper, AlbumApprove> implements BaseService<AlbumApprove> {

    @Resource
    private AlbumApproveMapper albumApproveMapper;

    @Autowired
    private AlbumVersionMapper albumVersionMapper;

    @Autowired
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private AlbumApprovalNodeMapper albumApprovalNodeMapper;

    @Autowired
    private AlbumCategoryMapper albumCategoryMapper;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<AlbumApprove> getRepository() {
        return albumApproveMapper;
    }

    /**
     * PC端根据知识专辑ID获取审批流程列表数据
     *
     * @param albumId
     * @param versionId
     * @return
     */
    public AlbumApproveFlowVo approvals(String albumId, String versionId) {

        // TODO: 2023/2/6  专辑版本审核流程模板
        AlbumApproveFlowVo result = new AlbumApproveFlowVo();
        //根据Id获取专辑信息
        AlbumInfo albumInfo = albumInfoMapper.selectById(albumId);
        //根据专辑Id查询所有版本数据
        List<AlbumVersion> albumVersionList = albumVersionMapper.
                selectList(new LambdaQueryWrapper<AlbumVersion>().eq(AlbumVersion::getAlbumId, albumId));
        //获取登录人信息
        String userId = userService.queryLoginUser();
        //判断当前登录人是否为专辑负责人(是 显示最新版本的审批流程 不是 显示最新生效中的版本的审批流程)
        if (userId.equals(albumInfo.getOwnerId())) {
            //判断是否有在审核中的版本
            List<AlbumVersion> albumVersionFirst = albumVersionList.stream().
                    filter(albumVersion -> albumVersion.getState() == 1).collect(Collectors.toList());

            if (CollectionUtils.isEmpty(albumVersionFirst)) {

            } else {
                albumVersionFirst.forEach(e -> {
                    //查询审核流程
                    QueryWrapper<AlbumApprove> queryWrapper = new QueryWrapper<>();
                    queryWrapper.eq("ALBUM_ID", albumId);
                    queryWrapper.eq("VERSION_ID", e.getPkId());
                    List<AlbumApprove> albumApproveList = albumApproveMapper.selectList(queryWrapper);
                    List<AlbumApproveVo> albumApproveVoList = albumApproveList.stream().map(albumApprove -> {
                        AlbumApproveVo albumApproveVo = new AlbumApproveVo();
                        BeanUtils.copyProperties(albumApprove, albumApproveVo);
                        return albumApproveVo;
                    }).collect(Collectors.toList());
                    result.setFlows(albumApproveVoList);
                    result.setState(albumInfo.getState());
                });
            }
        }

        return null;
    }

    /**
     * 管理端审核列表
     *
     * @param queryFilter
     * @return
     */
    @Override
    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        String type = (String) queryFilter.getParams().get("type");
        //获取全部数据
        IPage page = albumApproveMapper.findAllAlbumApprove(queryFilter.getPage(), queryFilter.getParams(), params);;
        List<AlbumApprove> albumApproveList = page.getRecords();
        //根据专辑Id进行数据分组
        Map<String, List<AlbumApprove>> albumApproveMap = albumApproveList.stream().collect(Collectors.groupingBy(AlbumApprove::getAlbumId));
        // 返回结果集
        List<AlbumApproveListVo> albumApproveListVos = new ArrayList<>();
        // 遍历每组的数据
        Set<Map.Entry<String, List<AlbumApprove>>> entries = albumApproveMap.entrySet();
        for (Map.Entry<String, List<AlbumApprove>> entry : entries) {
            List<AlbumApprove> value = entry.getValue();
            for (int i = 0; i < value.size(); i++) {
                //给结果集赋值
                AlbumApproveListVo albumApproveListVo = new AlbumApproveListVo();
                albumApproveListVo.setPkId(value.get(i).getAlbumInfo().getPkId());
                albumApproveListVo.setAlbumName(value.get(i).getAlbumInfo().getName());
                String categoryId = value.get(i).getAlbumInfo().getCategoryId();
                AlbumCategory albumCategory = albumCategoryMapper.selectById(categoryId);
                if (albumCategory.getGrade() == 1) {
                    albumApproveListVo.setCategoryName(albumCategory.getName());
                } else {
                    AlbumCategory albumCategoryParent = albumCategoryMapper.
                            selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
                    albumApproveListVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
                }
                albumApproveListVo.setOwnerName(userService.queryOsUserDto(value.get(i).getAlbumInfo().getOwnerId()).getFullName());
                albumApproveListVo.setCreateTime(value.get(i).getAlbumVersion().getCreateTime());
                albumApproveListVo.setApprovalStateName(AlbumVersionApproveEnum.getMessage(value.get(i).getAlbumVersion().getApprovalState()));
                albumApproveListVo.setApproverId(value.get(i).getApproverId());
                albumApproveListVo.setVersionId(value.get(i).getAlbumVersion().getPkId());
                albumApproveListVo.setApprovalId(value.get(i).getPkId());
                List<String> actionFlag = new ArrayList<>();
                if (value.get(i).getAlbumVersion().getApprovalState() == 1 || value.get(i).getAlbumVersion().getApprovalState() == 2) {
                    actionFlag.add("approve");
                } else {
                    actionFlag.add("view");
                }
                albumApproveListVo.setActionFlag(actionFlag);
                // 如果是最终审核通过,直接放入记录
                if (value.get(i).getAlbumVersion().getApprovalState() == 5) {
                    //实际审核人姓名和实际审核人审核时间
                    albumApproveListVo.setPreviousApproverName(value.get(i).getActualApproverName());
                    albumApproveListVo.setPreviousApproverTime(value.get(i).getApprovalTime());
                    albumApproveListVos.add(albumApproveListVo);
                    break;
                }
                // 判断审批结果是否为null(如果审批结果没有null，证明审批已结束;如果等于0，证明审批没通过)
                if (value.get(i).getResult() == null || value.get(i).getResult() == 0) {
                    if (i != 0) {
                        // 如果不是第一条数据,取前一条数据的实际审核人姓名
                        String actualApproverName = value.get(i - 1).getActualApproverName();
                        albumApproveListVo.setPreviousApproverName(actualApproverName);
                        // 如果不是第一条数据,取前一条数据的审批时间
                        Date approvalTime = value.get(i - 1).getApprovalTime();
                        albumApproveListVo.setPreviousApproverTime(approvalTime);
                    }
                    // 把该记录加到结果集中
                    albumApproveListVos.add(albumApproveListVo);
                    break;
                }
            }
        }
        if ("1".equals(type)){
            Page pageResult = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(), albumApproveListVos);
            return pageResult;
        } else {
            //获取当前登录人的userId和deptId
            String userId = userService.queryLoginUser();
            String deptId = userService.queryLoginUserDeptId();

            List<AlbumApproveListVo> albumApproveListVo1 = new ArrayList<>();
            //根据部门Id获取子部门信息
            //过滤数据
           albumApproveListVos.forEach(albumApproveListVo -> {
               Integer count = albumApprovalNodeMapper.selectAllByDeptId(deptId, albumApproveListVo.getApproverId());
               if (userId.equals(albumApproveListVo.getApproverId()) || count > 0) {
                    albumApproveListVo1.add(albumApproveListVo);
                }
            });
            Page pageResult = PageListUtils.getPages(queryFilter.getPage().getCurrent(), queryFilter.getPage().getSize(),albumApproveListVo1);
            return pageResult;
        }
    }

    /**
     * 获取当前登陆人待审核的专辑数量
     *
     * @return
     */
    public Integer total() {
        //获取当前登录人的userId和deptId
        String userId = userService.queryLoginUser();
        String deptId = userService.queryLoginUserDeptId();
        //查询全部审核节点的审核人Id
        List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeMapper.
                selectList(null);
        Set<String> approverIdSet = albumApprovalNodeList.parallelStream().
                map(AlbumApprovalNode::getApproverId).collect(Collectors.toSet());
        Integer total = 0;
        approverIdSet.forEach(approverId -> {
            //根据部门Id获取子部门信息
            Integer count = albumApprovalNodeMapper.selectAllByDeptId(deptId,approverId);
            //判断当前登录人是否有审核资格
            if (approverId.equals(userId) || count > 0) {

            }
        });
        return total;
    }

    /**
     * 对知识专辑审核中的版本进行审核(快速审核和批量审核)
     * @param albumApprovalRemark
     */
    @Transactional(rollbackFor = Exception.class)
    public void approve(AlbumApprovalRemark albumApprovalRemark) {
        List<String> approvalIdList = albumApprovalRemark.getPkIds();
        // 如果审批结论为null,置位空字符串
        if (StringUtils.isEmpty(albumApprovalRemark.getRemark())){
            albumApprovalRemark.setRemark("");
        }
        approvalIdList.forEach(approvalId -> {
            //插入审核结论
            AlbumApprove albumApprove = new AlbumApprove();
            albumApprove.setPkId(approvalId);
            albumApprove.setResult(albumApprovalRemark.getResult());
            albumApprove.setRemark(albumApprovalRemark.getRemark());
            albumApprove.setActualApprover(userService.queryLoginUser());
            albumApprove.setActualApproverName(userService.queryLoginFullName());
            albumApprove.setApprovalTime(new Date());
            albumApproveMapper.updateById(albumApprove);
            //修改版本状态
            AlbumApprove albumApproveById = albumApproveMapper.selectById(approvalId);
            AlbumApprove albumApproveByIdNext = albumApproveMapper.
                    selectOne(new QueryWrapper<AlbumApprove>().
                            eq("SORT", albumApproveById.getSort() + 1).eq("VERSION_ID",albumApproveById.getVersionId()));
            //如果审核节点不是终审节点且专辑审核未通过,修改专辑版本状态为节点驳回
            if (albumApproveById.getIsFinal() == 0 && albumApprovalRemark.getResult() == 0){
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                //专辑驳回
                albumVersion.setState(4);
                //专辑审核驳回
                albumVersion.setApprovalState(3);
                albumVersionMapper.updateById(albumVersion);
            }
            //如果审核节点是终审节点且专辑审核未通过,修改专辑版本状态为最终驳回
            if (albumApproveById.getIsFinal() == 1 && albumApprovalRemark.getResult() == 0){
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                //专辑驳回
                albumVersion.setState(4);
                //专辑审核驳回
                albumVersion.setApprovalState(4);
                albumVersionMapper.updateById(albumVersion);
            }
            //如果审核节点是终审节点且专辑审核通过,修改专辑版本状态为最终通过(即审核完成)
            if (albumApproveById.getIsFinal() == 1 && albumApprovalRemark.getResult() == 1){
                AlbumVersion albumVersion = new AlbumVersion();
                //更改其他专辑为未生效
                albumVersion.setState(3);
                albumVersionMapper.update(albumVersion,new QueryWrapper<AlbumVersion>().eq("ALBUM_ID",albumApproveById.getAlbumId()));
                //专辑生效(其他专辑未生效)
                albumVersion.setPkId(albumApproveById.getVersionId());
                albumVersion.setState(2);
                //专辑审核最终通过
                albumVersion.setApprovalState(5);
                albumVersionMapper.updateById(albumVersion);
            }
            // 如果当前节点的下个节点为终审节点通过且当前节点审核通过,修改状态为待最终审核
            if(albumApproveByIdNext != null && albumApproveByIdNext.getIsFinal() == 1 && albumApprovalRemark.getResult() == 1){
                AlbumVersion albumVersion = new AlbumVersion();
                albumVersion.setPkId(albumApproveById.getVersionId());
                albumVersion.setApprovalState(2);
                albumVersionMapper.updateById(albumVersion);
            }
        });

    }
}
