package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.BeanUtil;
import com.redxun.common.tool.IdGenerator;
import com.redxun.knowledge.album.entity.dao.AlbumCategory;
import com.redxun.knowledge.album.entity.dao.AlbumInfo;
import com.redxun.knowledge.album.entity.vo.AlbumCategoryVo;
import com.redxun.knowledge.album.mapper.AlbumCategoryMapper;
import com.redxun.knowledge.album.mapper.AlbumInfoMapper;
import com.redxun.knowledge.common.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* [业务--专辑分类表]业务服务类
*/
@Service
public class AlbumCategoryServiceImpl extends SuperServiceImpl<AlbumCategoryMapper, AlbumCategory> implements BaseService<AlbumCategory> {

    @Resource
    private AlbumCategoryMapper albumCategoryMapper;

    @Autowired
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<AlbumCategory> getRepository() {
        return albumCategoryMapper;
    }

    /**
     * 新增专辑分类
     * @param entity
     * @return
     */
    @Override
    public int insert(AlbumCategory entity) {
        if (BeanUtil.isEmpty(entity.getPkId())) {
            entity.setPkId(IdGenerator.getIdStr());
        }
        if (entity.getParent() == null || entity.getParent().isEmpty()){
            entity.setParent("0");
            entity.setGrade(1L);
        } else {
            entity.setGrade(2L);
        }
        //处理sort
        operatorSort(entity);
        //处理修改人(与框架填入冲突)
        entity.setUpdateBy(userService.queryLoginUser());
        return this.getRepository().insert(entity);
    }

    /**
     * 分页查询
     * @param queryFilter
     * @return
     */
    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        IPage page = albumCategoryMapper.findAllAlbumCategory(queryFilter.getPage(), queryFilter.getParams(), params);
        List<AlbumCategoryVo> records = page.getRecords();

        // TODO: 2023/2/1 根据分类名称查询
        Map<String, Object> param = queryFilter.getParams();
        if (param.containsKey("name")){

            // 孩子节点集合
            //List<String> childrenList = new ArrayList<>();
            //records.parallelStream().filter(e -> e.getGrade() == 1L).forEach(record -> {
            //    List<AlbumCategory> albumCategoryList = albumCategoryMapper.
            //            selectList(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getParent, record.getPkId()).orderByAsc(AlbumCategory::getSort));
            //    albumCategoryList.forEach(albumCategory -> {
            //        albumCategory.setOperator(userService.queryUser(record.getUpdateBy()).getFullName());
            //        childrenList.add(albumCategory.getPkId());
            //    });
            //    record.setOperator(userService.queryUser(record.getUpdateBy()).getFullName());
            //});
            return null;
        } else {
            //查询一级分类下的二级分类
            records.forEach(e -> {
                List<AlbumCategory> albumCategoryList = albumCategoryMapper.
                        selectList(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getParent, e.getPkId()).orderByAsc(AlbumCategory::getSort));
                List<AlbumCategoryVo> childAlbumCategoryVoList = albumCategoryList.parallelStream().map(albumCategory -> {
                    AlbumCategoryVo albumCategoryVo = new AlbumCategoryVo();
                    BeanUtils.copyProperties(albumCategory, albumCategoryVo);
                    albumCategoryVo.setOperator(userService.queryUser(albumCategoryVo.getUpdateBy()).getFullName());
                    return albumCategoryVo;
                }).collect(Collectors.toList());
                if (CollectionUtils.isEmpty(albumCategoryList)){
                    e.setChildren(null);
                } else {
                    e.setChildren(childAlbumCategoryVoList);
                }
                e.setOperator(userService.queryUser(e.getUpdateBy()).getFullName());
            });
            return page;
        }
    }

    /**
     * 专辑分类详情
     * @param pkId
     * @return
     */
    public AlbumCategoryVo info(String pkId) {
        AlbumCategoryVo albumCategoryVo = new AlbumCategoryVo();
        AlbumCategory albumCategory = albumCategoryMapper.selectById(pkId);
        BeanUtils.copyProperties(albumCategory,albumCategoryVo);
        return albumCategoryVo;
    }

    /**
     * 查询对应级别的专辑分类列表
     * @param level
     * @return
     */
    public List<AlbumCategoryVo> listOflevel(Integer level) {
        List<AlbumCategoryVo> albumCategoryVoList = albumCategoryMapper.listOflevel(level);
        return albumCategoryVoList;
    }

    public String del(String ids) {
        List<AlbumInfo> albumInfoList = albumInfoMapper.
                selectList(new QueryWrapper<AlbumInfo>().eq("CATEGORY_ID", ids));
        if (CollectionUtils.isNotEmpty(albumInfoList)){
            String result =
                    albumInfoList.parallelStream().map(AlbumInfo::getName).collect(Collectors.joining(","));
            return result;
        }
        AlbumCategory albumCategory = albumCategoryMapper.selectById(ids);
        // 查询该分类的下是否还有子分类
        List<AlbumCategory> albumCategoryList = albumCategoryMapper.
                selectList(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getParent, ids));
        if (!CollectionUtils.isEmpty(albumCategoryList)){
            return "1";
        }
        //变更sort(已删除数据为基准,后面数据sort向前变更)
        albumCategoryMapper.deleteBySort(albumCategory.getParent(),albumCategory.getSort());
        albumCategoryMapper.deleteByPkId(ids);
        return "-1";
    }

    /**
     * 编辑分类
     * @param albumCategory
     * @return
     */
    public String edit(AlbumCategory albumCategory) {
        if (albumCategory.getSort() == null){
            return "排序不能为空";
        }
        if (albumCategory.getParent().isEmpty()){
            albumCategory.setParent("0");
        }
        //修改层级
        QueryWrapper<AlbumCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("GRADE");
        queryWrapper.eq("PK_ID",albumCategory.getParent());
        AlbumCategory category = albumCategoryMapper.selectOne(queryWrapper);
        albumCategory.setGrade(category.getGrade() + 1);
        //变更sort
        operatorSort(albumCategory);
        //修改
        albumCategoryMapper.updateById(albumCategory);
        return albumCategory.getPkId();
    }


    /**
     * 新增和修改时进行排序处理
     * @param entity
     */
    private void operatorSort(AlbumCategory entity){
        List<Integer> sortList = albumCategoryMapper.selectSortList(entity.getParent());
        if (!CollectionUtils.isEmpty(sortList) && sortList.contains(entity.getSort())){
            //根据排序和父类Id变更排序
            albumCategoryMapper.insertOrUpdateBySort(entity.getParent(),entity.getSort());
        } else if (!CollectionUtils.isEmpty(sortList)){
            Integer lastSort = sortList.get(sortList.size() - 1);
            entity.setSort(lastSort + 1);
        } else {
            entity.setSort(1);
        }
    }

    /**
     * 通过上升、下降方式调整专辑分类顺序
     * @param pkId
     * @param direction
     */
    public void adjustDirection(String pkId, Integer direction) {
        AlbumCategory category = this.get(pkId);
        Integer sort = category.getSort();
        String parent = category.getParent();
        if (direction == 1){
            // 上升
            QueryWrapper<AlbumCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT",parent);
            queryWrapper.orderByAsc("SORT");
            List<AlbumCategory> albumCategoryList = albumCategoryMapper.selectList(queryWrapper);
            List<AlbumCategory> collect = albumCategoryList.stream().filter(e -> e.getSort() < sort).collect(Collectors.toList());
            AlbumCategory albumCategoryPre = collect.get(collect.size() - 1);
            // 交换位置
            int temp = sort;
            albumCategoryMapper.swapSort(pkId,albumCategoryPre.getSort());
            albumCategoryMapper.swapSort(albumCategoryPre.getPkId(),temp);
        } else {
            // 下降
            QueryWrapper<AlbumCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("PARENT",parent);
            queryWrapper.orderByAsc("SORT");
            List<AlbumCategory> albumCategoryList = albumCategoryMapper.selectList(queryWrapper);
            List<AlbumCategory> collect = albumCategoryList.stream().filter(e -> e.getSort() > sort).collect(Collectors.toList());
            AlbumCategory albumCategoryNext = collect.get(0);
            // 交换位置
            int temp = sort;
            albumCategoryMapper.swapSort(pkId,albumCategoryNext.getSort());
            albumCategoryMapper.swapSort(albumCategoryNext.getPkId(),temp);
        }
    }

    /**
     * 根据父Id查询子分类集合(创建专辑时PC端使用 懒加载模式)
     * @param pkId
     * @return
     */
    public List<AlbumCategory> lazyLoad(String pkId) {
        QueryWrapper<AlbumCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT",pkId);
        queryWrapper.orderByAsc("SORT");
        List<AlbumCategory> albumCategoryList = albumCategoryMapper.selectList(queryWrapper);
        albumCategoryList.forEach(e -> e.setHasChild(hasChild(e.getPkId())));
        return albumCategoryList;
    }

    /**
     * 判断是否有孩子节点(懒加载使用)
     * @param pkId
     * @return
     */
    private boolean hasChild(String pkId){
        QueryWrapper<AlbumCategory> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("PARENT",pkId);
        Integer count = albumCategoryMapper.selectCount(queryWrapper);
        return count > 0;
    }
}
