package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.dto.user.OsUserDto;
import com.redxun.knowledge.album.entity.dao.*;
import com.redxun.knowledge.album.entity.dto.AlbumApprovalNodeDto;
import com.redxun.knowledge.album.entity.dto.AlbumCatalogueDto;
import com.redxun.knowledge.album.entity.dto.AlbumInfoDto;
import com.redxun.knowledge.album.entity.enums.AlbumInfoEnum;
import com.redxun.knowledge.album.entity.enums.AlbumVersionEnum;
import com.redxun.knowledge.album.entity.vo.*;
import com.redxun.knowledge.album.mapper.*;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.service.KnowledgeCategoryServiceImpl;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import com.sun.xml.bind.v2.TODO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
* [业务--专辑基本信息表]业务服务类
*/
@Service
public class AlbumInfoServiceImpl extends SuperServiceImpl<AlbumInfoMapper, AlbumInfo> implements BaseService<AlbumInfo> {

    @Resource
    private AlbumInfoMapper albumInfoMapper;

    @Autowired
    private AlbumVersionMapper albumVersionMapper;

    @Autowired
    private AlbumCatalogueMapper albumCatalogueMapper;

    @Autowired
    private AlbumCatalogueKnowledgeMapper albumCatalogueKnowledgeMapper;

    @Autowired
    private AlbumCategoryMapper albumCategoryMapper;

    @Autowired
    private AlbumYelpMapper albumYelpMapper;

    @Autowired
    private AlbumApprovalNodeServiceImpl albumApprovalNodeService;

    @Autowired
    private AlbumYelpApprovalNodeServiceImpl albumYelpApprovalNodeService;

    @Autowired
    private AlbumApproveServiceImpl albumApproveService;

    @Autowired
    private KnowledgeServiceImpl knowledgeService;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Autowired
    private UserService userService;

    @Autowired
    private ThreadPoolExecutor threadPoolExecutor;

    @Override
    public BaseDao<AlbumInfo> getRepository() {
        return albumInfoMapper;
    }

    /**
     * 新增专辑
     * @param albumInfoDto 专辑基本信息&专辑目录&目录下收录知识
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public String create(AlbumInfoDto albumInfoDto) {
        AlbumInfo albumInfo = new AlbumInfo();
        BeanUtils.copyProperties(albumInfoDto,albumInfo);
        albumInfo.setPkId(IdGenerator.getIdStr());
        String userId = userService.queryLoginUser();
        // 获取当前登录人用户信息
        OsUserDto osUserDto = userService.queryOsUserDto(userId);
        // 加入用户信息数据
        albumInfo.setOwnerCompanyId(osUserDto.getCompanyId());
        albumInfo.setOwnerDepartmentId(osUserDto.getDeptId());
        albumInfo.setOwnerId(osUserDto.getUserId());
        albumInfo.setUpdateBy(osUserDto.getUserId());
        //新增专辑
        albumInfoMapper.insert(albumInfo);
        //生成一号版本
        AlbumVersion albumVersion = new AlbumVersion();
        albumVersion.setPkId(IdGenerator.getIdStr());
        albumVersion.setAlbumId(albumInfo.getPkId());
        albumVersion.setApplyUserId(osUserDto.getUserId());
        albumVersion.setSummary(albumInfoDto.getSummary());
        albumVersion.setUpdateBy(osUserDto.getUserId());
        albumVersionMapper.insert(albumVersion);
        //拉取专辑审核节点表
        List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeService.
                findAll(new QueryWrapper<AlbumApprovalNode>().orderByAsc("IS_FINAL", "CREATE_TIME_"));
        //将版本推送至专辑审核表中(各个节点都推送)
        AtomicInteger sort = new AtomicInteger(1);
        List<AlbumApprove> albumApproveList = albumApprovalNodeList.stream().map(albumApprovalNode -> {
            AlbumApprove albumApprove = new AlbumApprove();
            albumApprove.setPkId(IdGenerator.getIdStr());
            albumApprove.setAlbumId(albumInfo.getPkId());
            albumApprove.setVersionId(albumVersion.getPkId());
            albumApprove.setApproverId(albumApprovalNode.getApproverId());
            albumApprove.setApproverName(albumApprovalNode.getApproverName());
            albumApprove.setSort(sort.getAndIncrement());
            albumApprove.setIsFinal(albumApprovalNode.getIsFinal());
            albumApprove.setUpdateBy(userService.queryLoginUser());
            albumApprove.setName(albumApprovalNode.getName());
            return albumApprove;
        }).collect(Collectors.toList());
        albumApproveService.saveBatch(albumApproveList);
        // 向专辑目录表中添加记录
        if (CollectionUtils.isNotEmpty(albumInfoDto.getCatalogues())){
            List<AlbumCatalogueDto> catalogueDtoList = albumInfoDto.getCatalogues();
            AtomicInteger i = new AtomicInteger(1);
            catalogueDtoList.forEach(catalogueDto -> {
                AlbumCatalogue albumCatalogue = new AlbumCatalogue();
                albumCatalogue.setPkId(IdGenerator.getIdStr());
                albumCatalogue.setAlbumId(albumInfo.getPkId());
                albumCatalogue.setVersionId(albumVersion.getPkId());
                albumCatalogue.setName(catalogueDto.getName());
                albumCatalogue.setSort(i.getAndIncrement());
                albumCatalogue.setUpdateBy(osUserDto.getUserId());
                //向专辑目录知识表中添加记录
                if (CollectionUtils.isNotEmpty(catalogueDto.getContents())){
                    List<Long> contentIdList = catalogueDto.getContents();
                    contentIdList.forEach(contentId -> {
                        AlbumCatalogueKnowledge albumCatalogueKnowledge = new AlbumCatalogueKnowledge();
                        albumCatalogueKnowledge.setPkId(IdGenerator.getIdStr());
                        albumCatalogueKnowledge.setCatalogueId(albumCatalogue.getPkId());
                        albumCatalogueKnowledge.setKnowledgeId(contentId.toString());
                        albumCatalogueKnowledge.setUpdateBy(osUserDto.getUserId());
                        albumCatalogueKnowledgeMapper.insert(albumCatalogueKnowledge);
                    });
                }
                albumCatalogueMapper.insert(albumCatalogue);
            });
        }
        return albumInfo.getPkId();
    }

    public IPage query(QueryFilter queryFilter) {
        Map<String, Object> params = PageHelper.constructParams(queryFilter);
        //PC端条件过滤
        if (queryFilter.getParams().containsKey("scope")){
            String scope = (String) queryFilter.getParams().get("scope");
            Map<String, Object> map = new HashMap<>();
            if (scope.equals("1")){
                //我的专辑
                map.put("ownerId",userService.queryLoginUser());
                queryFilter.setParams(map);
            }else if (scope.equals("2")){
                // TODO: 2023/2/6 查看已发布的专辑和我自己的待审核的专辑
                //已发布的专辑
                map.put("state",2);
                queryFilter.setParams(map);
            }else if (scope.equals("3")){
                // TODO: 2023/2/3  我点评过的专辑
                map.put("yelp",1);
            }
        }
        IPage page = albumInfoMapper.findAllAlbumInfo(queryFilter.getPage(), queryFilter.getParams(), params);
        List<AlbumInfo> records = page.getRecords();
        List<AlbumInfoVo> albumInfoVoList = records.parallelStream().map(e -> {
            AlbumInfoVo albumInfoVo = new AlbumInfoVo();
            BeanUtils.copyProperties(e, albumInfoVo);
            //专辑分类信息
            CompletableFuture<Void> categoryCompletableFuture = CompletableFuture.runAsync(() -> {
                String categoryId = e.getCategoryId();
                AlbumCategory albumCategory = albumCategoryMapper.selectById(categoryId);
                if (albumCategory.getGrade() == 1) {
                    albumInfoVo.setCategoryName(albumCategory.getName());
                } else {
                    AlbumCategory albumCategoryParent = albumCategoryMapper.
                            selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
                    albumInfoVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
                }
            },threadPoolExecutor);
            //专辑收录知识量(当前生效版本各个目录中知识数量总和)
            CompletableFuture<Void> countKnowledgeCompletableFuture = CompletableFuture.runAsync(() -> {
                AlbumVersion albumVersion = albumVersionMapper.
                        selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                eq(AlbumVersion::getAlbumId, e.getPkId()).
                                eq(AlbumVersion::getState, 2));
                //没有生效的专辑(第一次创建专辑时的情况)
                if (albumVersion == null) {
                    AlbumVersion albumVersionFirst = albumVersionMapper.
                            selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                    eq(AlbumVersion::getAlbumId, e.getPkId()).
                                    eq(AlbumVersion::getState, 1));
                    if (albumVersionFirst == null){
                        AlbumVersion albumVersionFour = albumVersionMapper.
                                selectOne(new LambdaQueryWrapper<AlbumVersion>().
                                        eq(AlbumVersion::getAlbumId, e.getPkId()).
                                        eq(AlbumVersion::getState, 4));
                        Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersionFour.getPkId());
                        albumInfoVo.setKnowledgeTotal(countKnowledge);
                    } else {
                        Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersionFirst.getPkId());
                        albumInfoVo.setKnowledgeTotal(countKnowledge);
                    }
                } else {
                    Integer countKnowledge = albumInfoMapper.selectCountKnowledge(e.getPkId(), albumVersion.getPkId());
                    albumInfoVo.setKnowledgeTotal(countKnowledge);
                }
            },threadPoolExecutor);
            //专辑点评量(当前专辑中通过审核的点评量总和)
            CompletableFuture<Void> countYelpCompletableFuture = CompletableFuture.runAsync(() -> {
                Integer countYelp = albumYelpMapper.selectCount(new LambdaQueryWrapper<AlbumYelp>().
                        eq(AlbumYelp::getAlbumId, e.getPkId()).
                        eq(AlbumYelp::getState, 1));
                albumInfoVo.setYelpTotal(countYelp);
            },threadPoolExecutor);
            //其他参数
            CompletableFuture<Void> anotherCompletableFuture = CompletableFuture.runAsync(() -> {
                //专辑负责人姓名
                albumInfoVo.setOwnerName(userService.queryOsUserDto(e.getOwnerId()).getFullName());
                //专辑负责人头像
                albumInfoVo.setOwnerPhoto((String) userService.querySexAndPhoto(e.getOwnerId()).get("photo"));
                //专辑状态名称(0 显示专辑状态,不显示版本状态 1 显示版本状态)
                List<AlbumVersion> albumVersions = albumVersionMapper.selectList(new LambdaQueryWrapper<AlbumVersion>().
                        eq(AlbumVersion::getAlbumId, e.getPkId()).orderByDesc(AlbumVersion::getCreateTime));
                if (e.getState() == 0){
                    albumInfoVo.setStateName(AlbumInfoEnum.getMessage(e.getState()));
                } else {
                    albumInfoVo.setVersionState(albumVersions.get(0).getState());
                    albumInfoVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersions.get(0).getState()));
                }
                //操作人姓名及时间
                albumInfoVo.setOperator(userService.queryOsUserDto(e.getUpdateBy()).getFullName());
                albumInfoVo.setUpdateTime(e.getUpdateTime());
                //动作标记(M 管理端)
                List<String> actionFlagM = new ArrayList<>();
                List<String> actionFlag = new ArrayList<>();
                if (e.getState() == 0){
                    actionFlagM.add("delete");
                    //actionFlag.add("view");
                } else if (e.getState() == 1 && albumVersions.get(0).getState() == 1){
                    actionFlagM.add("view");
                    //actionFlag.add("view");
                    //actionFlag.add("edit");
                } else if (e.getState() == 1 && albumVersions.get(0).getState() == 2){
                    actionFlagM.add("edit");
                }else if (e.getState() == 1 && albumVersions.get(0).getState() == 4){
                    actionFlagM.add("view");
                    //actionFlag.add("delete");
                    //actionFlag.add("view");
                    //actionFlag.add("edit");
                }
                albumInfoVo.setActionFlagM(actionFlagM);
                albumInfoVo.setActionFlag(actionFlag);
            },threadPoolExecutor);
            CompletableFuture.allOf(
                    categoryCompletableFuture,
                    countKnowledgeCompletableFuture,
                    countYelpCompletableFuture,
                    anotherCompletableFuture).join();
            return albumInfoVo;
        }).collect(Collectors.toList());
        page.setRecords(albumInfoVoList);
        return page;
    }

    /**
     * 专辑详情
     * @param pkId
     * @return
     */
    public AlbumInfoDetailVo detail(String pkId) {
        AlbumInfoDetailVo albumInfoDetailVo = new AlbumInfoDetailVo();
        AlbumInfo albumInfo = albumInfoMapper.selectById(pkId);
        BeanUtils.copyProperties(albumInfo,albumInfoDetailVo);
        //专辑负责人姓名
        albumInfoDetailVo.setOwner(userService.queryOsUserDto(albumInfo.getOwnerId()).getFullName());
        //所属公司名称
        albumInfoDetailVo.setOwnerCompany(userService.findDeptByDeptId(albumInfo.getOwnerCompanyId()).getName());
        //所属部门名称
        albumInfoDetailVo.setOwnerDepartment(userService.findDeptByDeptId(albumInfo.getOwnerDepartmentId()).getName());
        //分类名称
        AlbumCategory albumCategory = albumCategoryMapper.selectById(albumInfo.getCategoryId());
        if (albumCategory.getGrade() == 1) {
            albumInfoDetailVo.setCategoryName(albumCategory.getName());
        } else {
            AlbumCategory albumCategoryParent = albumCategoryMapper.
                    selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
            albumInfoDetailVo.setCategoryName(albumCategoryParent.getName() + "|" + albumCategory.getName());
        }
        //状态名称
        albumInfoDetailVo.setStateName(AlbumInfoEnum.getMessage(albumInfo.getState()));
        //专辑点评量(当前专辑中通过审核的点评量总和)
        Integer countYelp = albumYelpMapper.selectCount(new LambdaQueryWrapper<AlbumYelp>().
                eq(AlbumYelp::getAlbumId, pkId).
                eq(AlbumYelp::getState, 1));
        albumInfoDetailVo.setYelpTotal(countYelp);
        //专辑版本信息
        QueryWrapper<AlbumVersion> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ALBUM_ID",pkId);
        queryWrapper.orderByDesc("CREATE_TIME_");
        List<AlbumVersion> albumVersions = albumVersionMapper.selectList(queryWrapper);
        //最新版本状态
        albumInfoDetailVo.setVersionState(albumVersions.get(0).getState());
        albumInfoDetailVo.setVersionStateName(AlbumVersionEnum.getMessage(albumVersions.get(0).getState()));
        //专辑版本集合
        List<AlbumVersionVo> albumVersionVoList = albumVersions.parallelStream().map(albumVersion -> {
            AlbumVersionVo albumVersionVo = new AlbumVersionVo();
            BeanUtils.copyProperties(albumVersion,albumVersionVo);
            //版本状态名称
            albumVersionVo.setStateName(AlbumVersionEnum.getMessage(albumVersion.getState()));
            return albumVersionVo;
        }).collect(Collectors.toList());
        albumInfoDetailVo.setVersions(albumVersionVoList);
        //专辑详情动作标记
        long count = albumVersions.stream().filter(e -> e.getState() == 2).count();
        List<String> actionFlag = new ArrayList<>();
        if (albumInfo.getState() == 1 && count > 0){
            actionFlag.add("edit");
            actionFlag.add("transfer");
            actionFlag.add("close");
        } else if (albumInfo.getState() == 0 && count > 0){
            actionFlag.add("edit");
            actionFlag.add("transfer");
            actionFlag.add("open");
        }
        albumInfoDetailVo.setActionFlag(actionFlag);
        return albumInfoDetailVo;
    }

    /**
     * 专辑版本详情
     * @param pkId
     * @return
     */
    public AlbumVersionDetailVo version(String pkId) {
        AlbumVersionDetailVo albumVersionDetailVo = new AlbumVersionDetailVo();
        AlbumVersion albumVersion = albumVersionMapper.selectById(pkId);
        albumVersionDetailVo.setPkId(albumVersion.getPkId());
        albumVersionDetailVo.setSummary(albumVersion.getSummary());
        //查询审核Id
        List<AlbumApprove> albumApproveLists = albumApproveService.
                findAll(new LambdaQueryWrapper<AlbumApprove>().
                        eq(AlbumApprove::getVersionId, pkId).
                        isNull(AlbumApprove::getResult).
                        orderByAsc(AlbumApprove::getSort));
        if (CollectionUtils.isNotEmpty(albumApproveLists)){
            albumVersionDetailVo.setApprovalId(albumApproveLists.get(0).getPkId());
        }
        //目录下知识的动作标记
        List<String> actionFlag = new ArrayList<>();
        if (albumVersion.getState() == 1){
            actionFlag.add("view");
        } else if (albumVersion.getState() == 2){
            actionFlag.add("delete");
            actionFlag.add("view");
        } else if (albumVersion.getState() == 3){
            actionFlag.add("view");
        }
        //审批流程节点
        QueryWrapper<AlbumApprove> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("ALBUM_ID",albumVersion.getAlbumId());
        queryWrapper.eq("VERSION_ID",pkId);
        queryWrapper.orderByAsc("SORT");
        List<AlbumApprove> albumApproveList = albumApproveService.findAll(queryWrapper);
        List<AlbumApproveVo> albumApproveVoList = new ArrayList<>();
        boolean flag = true;
        for (int i = 0; i < albumApproveList.size(); i++) {
            AlbumApproveVo albumApproveVo = new AlbumApproveVo();
            BeanUtils.copyProperties(albumApproveList.get(i),albumApproveVo);
            albumApproveVo.setComment(albumApproveList.get(i).getRemark());
            if (albumApproveList.get(i).getResult() == null){
                if (flag){
                    albumApproveVo.setApprovalStatus("提交审核");
                    flag = false;
                } else {
                    albumApproveVo.setApprovalStatus("");
                }
            } else if (albumApproveList.get(i).getResult() == 0){
                albumApproveVo.setApprovalStatus("审核驳回");
                flag = false;
            } else if (albumApproveList.get(i).getResult() == 1){
                albumApproveVo.setApprovalStatus("审核通过");
            }
            albumApproveVoList.add(albumApproveVo);
        }
        albumVersionDetailVo.setApprovalFlows(albumApproveVoList);
        // TODO: 2023/2/3  点评信息
        return albumVersionDetailVo;
    }

    /**
     * 批量更新审核流程节点数据
     * @param albumApprovalNodeDtoList
     */
    @Transactional(rollbackFor = Exception.class)
    public void batchUpdateApproveFlows(List<AlbumApprovalNodeDto> albumApprovalNodeDtoList) {
        //专辑审核节点插入
        List<AlbumApprovalNodeDto> albumApprovalNodeDto1 = albumApprovalNodeDtoList.
                stream().filter(e -> e.getType() == 1).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(albumApprovalNodeDto1)) {
            //清除原有数据
            albumApprovalNodeService.deleteAll();
            //批量插入数据
            List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeDto1.parallelStream().map(e -> {
                AlbumApprovalNode albumApprovalNode = new AlbumApprovalNode();
                albumApprovalNode.setPkId(IdGenerator.getIdStr());
                BeanUtils.copyProperties(e, albumApprovalNode);
                return albumApprovalNode;

            }).collect(Collectors.toList());
            albumApprovalNodeService.saveBatch(albumApprovalNodeList);
        }

        //点评审核节点插入
        List<AlbumApprovalNodeDto> albumApprovalNodeDto2 = albumApprovalNodeDtoList.
                stream().filter(e -> e.getType() == 2).collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(albumApprovalNodeDto2)) {
            //清除原有数据
            albumYelpApprovalNodeService.deleteAll();
            //批量插入数据
            List<AlbumYelpApprovalNode> albumYelpApprovalNodeList = albumApprovalNodeDto2.parallelStream().map(e -> {
                AlbumYelpApprovalNode albumYelpApprovalNode = new AlbumYelpApprovalNode();
                albumYelpApprovalNode.setPkId(IdGenerator.getIdStr());
                BeanUtils.copyProperties(e, albumYelpApprovalNode);
                return albumYelpApprovalNode;
            }).collect(Collectors.toList());
            albumYelpApprovalNodeService.saveBatch(albumYelpApprovalNodeList);
        }
    }

    /**
     * 查询专辑审核节点表数据
     * @return
     */
    public List<AlbumApproverNodeVo> approveFlowsAlbumApprovalNode() {

        List<AlbumApprovalNode> albumApprovalNodeList = albumApprovalNodeService.findAll(null);
        List<AlbumApproverNodeVo> collect = albumApprovalNodeList.parallelStream().map(e -> {
            AlbumApproverNodeVo albumApproverNodeVo = new AlbumApproverNodeVo();
            BeanUtils.copyProperties(e, albumApproverNodeVo);
            return albumApproverNodeVo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * 查询点评审核节点表数据
     * @return
     */
    public List<AlbumApproverNodeVo> approveFlowsAlbumYelpApprovalNode() {
        List<AlbumYelpApprovalNode> albumYelpApprovalNodeList = albumYelpApprovalNodeService.findAll(null);
        List<AlbumApproverNodeVo> collect = albumYelpApprovalNodeList.parallelStream().map(e -> {
            AlbumApproverNodeVo albumApproverNodeVo = new AlbumApproverNodeVo();
            BeanUtils.copyProperties(e, albumApproverNodeVo);
            return albumApproverNodeVo;
        }).collect(Collectors.toList());
        return collect;
    }

    /**
     * PC端根据知识专辑ID获取知识专辑详情
     * @param pkId
     * @return
     */
    public AlbumInfoDetailPcVo info(String pkId) {
        //返回结果
        AlbumInfoDetailPcVo albumInfoDetailPcVo = new AlbumInfoDetailPcVo();
        //查询专辑详情
        AlbumInfo albumInfo = this.get(pkId);
        //专辑详情(封装数据)
        AlbumInfoDetailVo albumInfoDetailVo = detail(pkId);
        BeanUtils.copyProperties(albumInfoDetailVo,albumInfoDetailPcVo);
        albumInfoDetailPcVo.setVersions(new ArrayList<>());
        //专辑分类赋值
        AlbumCategory albumCategory = albumCategoryMapper.selectById(albumInfo.getCategoryId());
        if (albumCategory.getGrade() == 1) {
            albumInfoDetailPcVo.setCategoryLevel1Id(albumCategory.getPkId());
            albumInfoDetailPcVo.setCategoryLevel1(albumCategory.getName());
        } else {
            AlbumCategory albumCategoryParent = albumCategoryMapper.
                    selectOne(new LambdaQueryWrapper<AlbumCategory>().eq(AlbumCategory::getPkId, albumCategory.getParent()));
            albumInfoDetailPcVo.setCategoryLevel2Id(albumCategoryParent.getPkId());
            albumInfoDetailPcVo.setCategoryLevel2(albumCategoryParent.getName());
        }
        //判断登录人是否是专辑所属人(是 查看审核中的版本(没有审核中的,查看已发布的) 不是 查看已发布的版本)
        String userId = userService.queryLoginUser();
        if (albumInfo.getOwnerId().equals(userId)){
            //查询版本信息(审核中)
            AlbumVersion albumVersion1 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 1));
            //查询版本信息(驳回)
            AlbumVersion albumVersion4 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 4));
            if (albumVersion1 != null){
                albumInfoDetailPcVo.setSummary(albumVersion1.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion1.getPkId());
                albumInfoDetailPcVo.setState(albumVersion1.getState());
                albumInfoDetailPcVo.setStateName(AlbumInfoEnum.getMessage(albumVersion1.getState()));
            } else if (albumVersion4 != null){
                albumInfoDetailPcVo.setSummary(albumVersion4.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion4.getPkId());
                albumInfoDetailPcVo.setState(albumVersion4.getState());
                albumInfoDetailPcVo.setStateName(AlbumInfoEnum.getMessage(albumVersion4.getState()));
            } else {
                //生效中的专辑
                AlbumVersion albumVersion2 = albumVersionMapper.
                        selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 2));
                albumInfoDetailPcVo.setSummary(albumVersion2.getSummary());
                albumInfoDetailPcVo.setVersionId(albumVersion2.getPkId());
                albumInfoDetailPcVo.setState(albumVersion2.getState());
                albumInfoDetailPcVo.setStateName(AlbumInfoEnum.getMessage(albumVersion2.getState()));
            }
            albumInfoDetailPcVo.setShowStatus(1);
            List<String> actionFlag = new ArrayList<>();
            actionFlag.add("edit");
            albumInfoDetailPcVo.setActionFlag(actionFlag);
        } else {
            //不是专辑所属人,查看已发布的版本信息
            AlbumVersion albumVersion2 = albumVersionMapper.
                    selectOne(new QueryWrapper<AlbumVersion>().eq("ALBUM_ID", pkId).eq("STATE", 2));
            if (albumVersion2 == null){
                albumInfoDetailPcVo.setShowStatus(0);
                return albumInfoDetailPcVo;
            }
            albumInfoDetailPcVo.setSummary(albumVersion2.getSummary());
            albumInfoDetailPcVo.setVersionId(albumVersion2.getPkId());
            albumInfoDetailPcVo.setState(albumVersion2.getState());
            albumInfoDetailPcVo.setStateName(AlbumInfoEnum.getMessage(albumVersion2.getState()));
            albumInfoDetailPcVo.setShowStatus(0);
        }
        return albumInfoDetailPcVo;
    }
}
