
package com.redxun.knowledge.map.service;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.map.entity.dao.Map;
import com.redxun.knowledge.map.entity.dao.MapContent;
import com.redxun.knowledge.map.entity.dao.MapGroupPurviewUser;
import com.redxun.knowledge.map.entity.dao.MapRoute;
import com.redxun.knowledge.map.entity.vo.MapContentVo;
import com.redxun.knowledge.map.mapper.MapContentMapper;
import com.redxun.knowledge.map.mapper.MapGroupPurviewUserMapper;
import com.redxun.knowledge.map.mapper.MapMapper;
import com.redxun.knowledge.map.mapper.MapRouteMapper;
import com.redxun.knowledge.mapper.KnowledgeMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;
import javax.annotation.Resource;

/**
 * [业务--知识地图学习路径表]业务服务类
 */
@Service
public class MapRouteServiceImpl extends SuperServiceImpl<MapRouteMapper, MapRoute> implements BaseService<MapRoute> {

    @Resource
    private MapRouteMapper mapRouteMapper;

    @Autowired
    private MapMapper mapMapper;

    @Autowired
    private UserService userService;

    @Autowired
    private MapContentMapper mapContentMapper;

    @Autowired
    private MapGroupPurviewUserMapper mapGroupPurviewUserMapper;

    @Autowired
    private KnowledgeMapper knowledgeMapper;

    @Autowired
    private MapServiceImpl mapService;

    @Override
    public BaseDao<MapRoute> getRepository() {
        return mapRouteMapper;
    }

    /**
     * PC端根据知识地图路径ID获取知识地图学习内容，根据tops参数设置请求内容条数。(权限判断)
     *
     * @param routeId
     * @param tops
     * @return
     */
    public List<MapContentVo> contentTops(String routeId, Integer tops) {
        // 获取地图
        Map map = mapMapper.selectById(mapRouteMapper.selectById(routeId).getMapId());
        String userId = userService.queryLoginUser();
        if (mapService.hasRole(map.getPkId())) {
            List<MapContent> mapContentList = mapContentMapper.selectList(new LambdaQueryWrapper<MapContent>().eq(MapContent::getRouteId, routeId).orderByAsc(MapContent::getSort));
            List<MapContentVo> result = mapContentList.stream().limit(tops).map(mapContent -> {
                MapContentVo mapContentVo = new MapContentVo();
                BeanUtils.copyProperties(mapContent, mapContentVo);
                Integer count = mapService.updateState(userId, mapContent.getMapId(), mapContent.getRouteId(), mapContent.getPkId());
                if (count > 0) {
                    mapContentVo.setState(1);
                } else {
                    mapContentVo.setState(0);
                }
                return mapContentVo;
            }).collect(Collectors.toList());
            return result;
        }
        return null;
    }
}
