
package com.redxun.knowledge.service;

import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.datasource.DataSourceContextHolder;
import com.redxun.dto.user.OsUserDto;
import com.redxun.feign.sys.SystemClient;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.CommonEntity;
import com.redxun.knowledge.entity.vo.DicVo;
import com.redxun.knowledge.entity.vo.PositionSequenceVo;
import com.redxun.knowledge.mapper.CommonMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 业务系统相关服务类
 */
@Slf4j
@Service
public class CommonServiceImpl extends SuperServiceImpl<CommonMapper, CommonEntity> implements BaseService<CommonEntity> {

    @Resource
    private CommonMapper commonMapper;

    @Autowired
    private UserService userService;

    @Resource
    @Lazy
    SystemClient systemClient;

    @Override
    public BaseDao<CommonEntity> getRepository() {
        return commonMapper;
    }

    /**
     * 查询值级列表
     *
     * @return
     */
    public List<String> queryPositionGrade() {
        DataSourceContextHolder.setDataSource("train");
        List<String> result = commonMapper.queryPositionGrade();

        // 调用完成后清除当前数据源
        DataSourceContextHolder.clearDataSource();

        return result;
    }

    public List<PositionSequenceVo> queryPositionSequence() {
        DataSourceContextHolder.setDataSource("train");
        List<PositionSequenceVo> result = commonMapper.queryPositionSequence();
        // 调用完成后清除当前数据源
        DataSourceContextHolder.clearDataSource();
        return result;
    }

    /**
     * 获取字典数据(知识地图类型)
     * @param alias
     * @return
     */
    public List<DicVo> queryDic(String alias) {
        List<DicVo> result = commonMapper.queryDic(alias);
        return result;
    }

    /**
     * 获取当前登录用户信息
     * @return
     */
    public Map<String, Object> loginUserInfo() {
        String userId = userService.queryLoginUser();
        OsUserDto osUserDto = userService.queryOsUserDto(userId);
        Map<String, Object> map = new HashMap<>();
        map.put("userId",userId);
        map.put("name",osUserDto.getFullName());
        map.put("companyName",null);
        map.put("deptName",osUserDto.getDeptName());
        return map;
    }
}
